/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#ifndef __operator_FollicleOp_h__
#define __operator_FollicleOp_h__

namespace fe
{

/**************************************************************************//**
	@brief Operator to edit surface locations

	@ingroup operator
*//***************************************************************************/
class FE_DL_EXPORT FollicleOp:
	public OperatorState,
	public Initialize<FollicleOp>
{
	public:

						FollicleOp(void):
							m_lastX(-1),
							m_lastY(-1),
							m_lastUV(-1,-1),
							m_brushed(FALSE),
							m_dragging(FALSE),
							m_areaSelect(FALSE)								{}

virtual					~FollicleOp(void)									{}

		void			initialize(void);

						//* As HandlerI
virtual	void			handle(Record& a_rSignal);

	protected:

		void			addFollicle(const SpatialVector& a_intersection);

		void			drawFollicle(sp<DrawI>& a_rspDraw,
								const SpatialVector& a_rPoint,
								Real a_rimSize,
								BWORD a_moved,BWORD a_added,
								const Color& a_rColorCenter,
								const Color& a_rColorRim);

		Record			getEdit(I32 a_index);
		void			movePicked(const SpatialVector& a_rDelta);
		void			slideOnUV(I32 a_index,const Vector2 a_deltaUV);
		void			snapToSurface(I32 a_index);

		BWORD			pickToggle(I32 a_index);
		Real			picked(I32 a_index);
		void			pick(I32 a_index,Real a_weight=1.0);

		BWORD			hideToggle(I32 a_index);
		BWORD			hidden(I32 a_index);
		void			hide(I32 a_index);

		BWORD			moved(I32 a_index);

virtual	void			setupState(void);
virtual	BWORD			loadState(const String& rBuffer);

	private:
	class Pick
	{
		public:
					Pick(I32 a_index,Real a_weight):
						m_index(a_index),
						m_weight(a_weight)									{}

			I32		m_index;
			Real	m_weight;
	};

		sp<SurfaceAccessorI>		m_spInputPoint;
		sp<SurfaceAccessorI>		m_spOutputPoint;
		sp<SurfaceAccessorI>		m_spOutputNormal;
		sp<SurfaceAccessorI>		m_spHideGroup;
		sp<SurfaceI>				m_spDriver;
		WindowEvent					m_event;

		sp<DrawMode>				m_spWireframe;
		sp<DrawMode>				m_spSolid;

		sp< RecordMap<I32> >		m_spEditMap;
		Accessor<I32>				m_aIndex;
		Accessor<SpatialVector>		m_aLocation;
		Accessor<Vector2>			m_aUV;

		BWORD						m_brushed;
		I32							m_lastX;
		I32							m_lastY;
		Vector2						m_lastUV;

		std::vector<Color>			m_colorRim;
		std::vector<Color>			m_colorCenter;
		std::vector<Pick>			m_pickArray;
		BWORD						m_dragging;
		BWORD						m_areaSelect;
};

} /* namespace */

#endif /* __operator_FollicleOp_h__ */
